import uuid

from typing import List

import barker.schemas.modifier as schemas

from fastapi import APIRouter, Depends, HTTPException, Security, status
from sqlalchemy import select
from sqlalchemy.exc import SQLAlchemyError

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.modifier import Modifier
from ..schemas.user_token import UserToken


router = APIRouter()


@router.post("", response_model=schemas.Modifier)
def save(
    data: schemas.ModifierIn,
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    try:
        with SessionFuture() as db:
            item = Modifier(
                name=data.name,
                show_in_bill=data.show_in_bill,
                price=data.price,
                modifier_category_id=data.modifier_category.id_,
                is_active=data.is_active,
            )
            db.add(item)
            db.commit()
            return modifier_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.put("/{id_}", response_model=schemas.Modifier)
def update_route(
    id_: uuid.UUID,
    data: schemas.ModifierIn,
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    try:
        with SessionFuture() as db:
            item: Modifier = db.execute(select(Modifier).where(Modifier.id == id_)).scalar_one()
            item.name = data.name
            item.show_in_bill = data.show_in_bill
            item.price = data.price
            item.modifier_category_id = data.modifier_category.id_
            item.is_active = data.is_active
            db.commit()
            return modifier_info(item)
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.delete("/{id_}", response_model=schemas.ModifierBlank)
def delete_route(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.ModifierBlank:
    try:
        with SessionFuture() as db:
            item: Modifier = db.execute(select(Modifier).where(Modifier.id == id_)).scalar_one()
            db.delete(item)
            db.commit()
            return modifier_blank()
    except SQLAlchemyError as e:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail=str(e),
        )


@router.get("", response_model=schemas.ModifierBlank)
def show_blank(
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.ModifierBlank:
    return modifier_blank()


@router.get("/list", response_model=List[schemas.Modifier])
def show_list(user: UserToken = Depends(get_user)) -> List[schemas.Modifier]:
    with SessionFuture() as db:
        return [modifier_info(item) for item in db.execute(select(Modifier).order_by(Modifier.name)).scalars().all()]


@router.get("/{id_}", response_model=schemas.Modifier)
def show_id(
    id_: uuid.UUID,
    user: UserToken = Security(get_user, scopes=["modifiers"]),
) -> schemas.Modifier:
    with SessionFuture() as db:
        item: Modifier = db.execute(select(Modifier).where(Modifier.id == id_)).scalar_one()
        return modifier_info(item)


def modifier_info(item: Modifier) -> schemas.Modifier:
    return schemas.Modifier(
        id=item.id,
        name=item.name,
        showInBill=item.show_in_bill,
        price=item.price,
        isActive=item.is_active,
        modifierCategory=schemas.ModifierCategoryLink(
            id=item.modifier_category.id,
            name=item.modifier_category.name,
        ),
    )


def modifier_blank() -> schemas.ModifierBlank:
    return schemas.ModifierBlank(
        name="",
        showInBill=False,
        price=0,
        isActive=True,
    )
