from datetime import date, timedelta
from typing import List

from fastapi import APIRouter, Depends, Security
from sqlalchemy import func, select
from sqlalchemy.orm import Session

from ...core.config import settings
from ...core.security import get_current_active_user as get_user
from ...db.session import SessionFuture
from ...models.inventory import Inventory
from ...models.kot import Kot
from ...models.tax import Tax
from ...models.voucher import Voucher
from ...models.voucher_type import VoucherType
from ...schemas.tax_report import TaxReport, TaxReportItem
from ...schemas.user_token import UserToken
from . import check_audit_permission, report_finish_date, report_start_date


router = APIRouter()


@router.get("", response_model=TaxReport)
def get_tax_report(
    start_date: date = Depends(report_start_date),
    finish_date: date = Depends(report_finish_date),
    user: UserToken = Security(get_user, scopes=["tax-report"]),
) -> TaxReport:
    check_audit_permission(start_date, user.permissions)
    with SessionFuture() as db:
        return TaxReport(
            startDate=start_date.strftime("%d-%b-%Y"),
            finishDate=finish_date.strftime("%d-%b-%Y"),
            amounts=get_tax(start_date, finish_date, db),
        )


def get_tax(s: date, f: date, db: Session) -> List[TaxReportItem]:
    start_date = s + timedelta(minutes=settings.NEW_DAY_OFFSET_MINUTES)
    finish_date = f + timedelta(days=1, minutes=settings.NEW_DAY_OFFSET_MINUTES)

    amounts = db.execute(
        select(
            Tax.name,
            Inventory.tax_rate,
            func.coalesce(func.sum(Inventory.net), 0),
            func.coalesce(func.sum(Inventory.tax_amount), 0),
        )
        .join(Voucher.kots)
        .join(Kot.inventories)
        .join(Inventory.tax)
        .where(
            Voucher.date >= start_date,
            Voucher.date <= finish_date,
            Voucher.voucher_type == VoucherType.REGULAR_BILL,
        )
        .group_by(Tax.name, Inventory.tax_rate)
        .order_by(Tax.name, Inventory.tax_rate)
    ).all()
    return [TaxReportItem(name=f"{i[0]} - {i[1]:.2%}", taxRate=i[1], saleAmount=i[2], amount=i[3]) for i in amounts]
